# README #

inMOTIFin is a simulation software to generate sequences with motif instances inserted in them.

### What is this repository for? ###

* motif instances sampled from PPMs
* motifs are generated from a Dirichlet prior (easy motifs with higher information content, see [here](docs/motif_simulation.md))
* background sequence is uniform randomly sampled from ACTG (base probabilities are provided by the user)
* motif instances inserted into a background sequence by replacing the background with the motif instance
* any number of motif is inserted per background and overlap is not allowed
* instance from the same motif can be inserted multiple times
* total number of sequences can be set
* same background can be selected more than once
* each motif have a probability of insertion (provided by user, per sequence or overall)
* location of the insertion is uniform randomly selected within the length of the background
* final sequences returned in a fasta file
* locations of the insertion (index) is returned as bed (zero based, end coordinate not included): seqname + location_start + location_end + orientation + motifname_instanceseq
* upon request, return the actual number of occurence per motif and a list of instances for each motif

* PFMs fetched from JASPAR database by JASPAR ID (base id + version) and normalized so that we have probabilities
* user can provide PFM (jaspar format) or PPM (meme format)
* background is provided by the user
* background sequences should be provided in fasta format

* motif instances inserted into a background sequence by adding new bases (total length increases with each motif)
* any number of motif is inserted per background and overlap is allowed


The simulation is powered by [DagSim](https://uio-bmi.github.io/dagsim/quickstart.html)

The directec acyclic graph (DAG) of the simulation is shown below:

![DAG of the simulation](docs/dag.png)

### How do I get set up? ###

* Summary of set up

You can either install from [testpypi](https://test.pypi.org/project/inMOTIFin/) with the command

```python -m  pip install -i https://test.pypi.org/simple/ --extra-index-url https://pypi.org/simple inMOTIFin```

or pull the container at [dockerhub](https://hub.docker.com/r/cbgr/inmotifin/tags). If you have docker installed, you can run from within the docker container with

```sudo docker run -it cbgr/inmotifin:0.2.0 bash ```

Altenarively, you can also clone this repository, you may install the package locally by `pip install -e . ` run in the folder where the `setup.py` file resides.

* Dependencies

The code is written in python 3.9 . Dependencies ([numpy](https://numpy.org/), [DagSim](https://uio-bmi.github.io/dagsim/quickstart.html), [pyjaspar](https://github.com/asntech/pyjaspar), and [biopython](https://biopython.org/)) are installed with the software.

* How to run tests

Unit tests can be run with the `pytest . ` command after cloning and installing the package locally.

As a developer, you can also see the results of the unit tests and an intergation test at pipelines.


### Contribution guidelines ###

* Coding style

 The code is written in object oriented way. The code is documented with docstring following the numpy style. Please add input and output types to each function. Please set linting with pylint and flake8.

* Writing tests

The aim is to have a decent test coverage, such as your best effort. The test folder mirrors the structure of the source (inmotifin) folder. The tests are using the unittest framework.

If you are not sure what to test, one way to go about it is to find which lines of the code are covered. This can be done using [coverage.py](https://coverage.readthedocs.io/en/latest/cmd.html) `coverage run -m pytest && coverage html` command. 
This generates multiple outputs, the one to keep an eye on is `htmlcov/index.html`. When opening this in your browser you can check which lines are covered by test and which are not.

* Code review

Upon pull request. Please do not merge to main/dev without contacting the authors.

* Other guidelines

The repository is connected to Jira, where the features and issues are listed. After familiarizing yourself with the current status of the software and the style, pick a feature and (from dev) create a branch for it in Jira. Please work on that branch and use smart commits. Once the feature is added, linting and the unit tests are passing, open a pull request to dev.

### Who do I talk to? ###

* Repo owner or admin

Kata Ferenc (k.t.ferenc@ncmm.uio.no)

* Other community or team contact

Anthony Mathelier (anthony.mathelier@ncmm.uio.no)

### License
[MIT license](https://opensource.org/license/mit)